<?php
/**
 * Defines the CryptDES encryption class.
 *
 * @package CryptLib
 */

/**
 * This class supports the standard DES encryption algorithm.
 * You can supply an optional 2 character salt to use during the
 * encryption.  If not supplied, the system will auto-generate
 * a default salt value.
 * 
 * @package CryptLib
 * @version 0.1
 * @author Randy L. Pearson
 */
class CryptDES implements CryptEngine
{
	private $options;
	
	/**
	 * Constructor
	 * 
	 * @param PropertyHolder $options Optional list of properties.  Extracts the 'salt' property if found.
	 * @return none Throws error if encryption algorithm not available or bad 'salt' property.
	 */
	public function __construct(PropertyHolder $options = null)
	{
		if (CRYPT_STD_DES != 1) {
			throw new Exception("Standard DES encryption is not available on this system.");			
		}

		if ($options->isValid("salt")) {
			if (strlen($options->salt) != 2) {
				throw new Excpetion("Standard DES requires a 2 character salt.");
			} else {
				$this->options = new PropertyBag(array("salt" => $options->salt));
			}
		} else {
			$this->options = new PropertyBag();
		}
	}
	
	/**
	 * Encrypt the string using the Standard DES algorithm.  Uses 'salt' string if provided.
	 * 
	 * @param string $data Data to be encrypted.
	 * @return string The encrypted data.
	 */
	public function encrypt($data)
	{
		
		if ($this->options->isValid("salt")) {
			return crypt($data, $this->options->salt);
		} else {
			return crypt($data);
		}
	}
	
	/**
	 * Standard DES is a one-way encryption, so throws error always.
	 */
	public function decrypt($data)
	{
		throw new Exception("Standard DES is a one-way encryption.  Cannot decrypt data.");
	}
	
	/**
	 * Changes the current options for the encryption scheme.  Only looks for the 'salt' property.
	 * 
	 * @param PropertyHolder $options The properties to be copied.  Only copies 'salt' if found.
	 * @return none Throws error if 'salt' property is not found or invalid.
	 */
	public function setOptions(PropertyHolder $options)
	{
		if ($options->isValid("salt")) {
			if (strlen($options->salt) != 2) {
				throw new Exception("Standard DES requires a 2 character salt.");
			}	
		
			if ($this->options->isValid("salt"))
				$this->options->salt = $options->salt;
			else
				$this->options->add("salt", $options->salt);
		} else {
			throw new Exception("Missing 'salt' property in options");
		}
	}
	
	/**
	 * Returns the current options.
	 * 
	 * @return PropertyHolder A list of current properties for the engine.
	 */
	public function getOptions()
	{
		return $this->options;
	}
}

?>
