<?php
/**
 * Defines the abstract Field class.
 * 
 * @package FormLib 
 */

/**
 * Base class used to derive other types of HTML fields.
 * 
 * This version adds the optional 'style' property that can be used
 * to customize the appearance of the field.
 * 
 * @package FormLib
 * @version 0.4
 * @author Randy L. Pearson
 */

abstract class Field
{
	protected $properties;
	
	/**
	 * Constructor
	 * 
	 * Initializes a Field object.
	 * 
	 * @param string $name Name assigned to the field.
	 * @param string $label Label (e.g. text) assigned to the field.
	 * @param string $value Value assigned to the field.
	 * @param Validator $validator Object that can check the
	 * field's data for validity
	 * @param PropertyHolder $style Additional style hinting information.
	 */
	public function __construct($name, $label, $value = "", $validator = false, $style = false)
	{
		$this->properties = new PropertyBag(
				array(
					"name" => $name,
					"label" => $label,
					"value" => $value,
					"validator" => false,
					"style" => false,
					"validated" => false
					));

		if ($validator)
			if ($validator instanceof Validator)
				$this->validator = $validator;
			else
				throw new Exception("Field __construct error: \$validator must be a Validator object, but it is a ".get_class($validator));
		
		if ($style)
			if ($style instanceof PropertyHolder)
				$this->style = $style;
			else
				throw new Exception("Field constructor error: \$style is of type (".get_class($style).") which does not implement the PropertyHolder interface.");
	}
	
	/**
	 * Property accessor.
	 * 
	 * @param string $name Name of the property to return.
	 */
	public function __get($name)
	{
		return $this->properties->$name;
	}
	
	/**
	 * Property mutator.
	 * 
	 * @param string $name Name of the property to change.
	 * @param any $value Value to assign to the property.
	 */
	public function __set($name,$value)
	{
		$this->properties->$name = $value;
	}

	/**
	 * Function to render the field as HTML.
	 * 
	 * This function assumes forms are laid out in a 2 column
	 * table.  It displays the field's label, then calls the
	 * abstract displayField() function.
	 * 
	 * @param PropertyHolder $formStyle Style hints from the containing form.
	 */
	public function display($formStyle)
	{
		echo "<tr>\n<td align=\"right\">\n";
		echo $this->label;
		echo " : </td>\n<td align=\"left\">";
		$this->displayField($formStyle);
		echo "</td>\n</tr>\n";
		if ($this->validated && !$this->isValid()) {
			echo "<tr><td colspan=\"2\" align=\"center\"><font color=\"red\">" .
				$this->validator->errorMessage($this) .
				"</font></td></tr>\n";
		}
	}

	/**
	 * Renders the field as HTML.  Must be overridden in child classes.
	 */	
	protected abstract function displayField($formStyle);
	
	/**
	 * Checks to see if the data entered into a field is valid.
	 * 
	 * @return boolean <b>true</b> if valid (or does not have a validator
	 * object)
	 * 
	 * @see Validator
	 * @see Validator::isValid()
	 */
	public function isValid()
	{
		$this->validated = true;
		if ($this->validator instanceof Validator)
			return $this->validator->isValid($this);
		else
			return true;
	}
}

?>
