<?php
/**
 * Defines the abstract Form class.
 * 
 * @package FormLib
 */
 
/**
 * Form class
 * 
 * This class is used to build an HTML form that consists of one
 * or more fields and submit buttons.  Normally not used directly,
 * but instead used as a base class.
 * 
 * Version 0.8 changes the static load() function to return the
 * form without calling the process() function.  That means users
 * of forms must do this themselves now.  However, it also means
 * that the form keeps its field values when successfully loaded.
 * 
 * Version 0.9 moved the code that outputs table row and cells here from the
 * Button class.
 * 
 * @package FormLib
 * @version 0.9
 * @author Randy L. Pearson
 */
abstract class Form
{
	/**
	 * Helper function that decodes the POST data and determines
	 * the name of the form, then creates the form and populates
	 * the values of all the fields automatically.  If a form
	 * was posted, it also invokes the process() method so the
	 * form can handle the data in whatever fashion it deems
	 * necessary.  Can you smell the polymorphism at work? 
	 * 
	 * Returns false if no form was posted.
	 * 
	 * @return Form An instance of a Form derived class.
	 */
	static function load()
	{
		if( empty($_POST) || !isset($_POST['form-name']) )
			return false;
			
		$form = new $_POST['form-name'];
		
		foreach($form->fields as $fld)
			if( isset($_POST[$fld->name]) )
				$fld->value = $_POST[$fld->name];
		
		return $form;
	}
	
	/**
	 * Derived form classes must implement this function.
	 * 
	 * @return Form Must return true if valthe form object ($this) or false on error.
	 */
	abstract public function process();

	/**
	 * Private variables.
	 */
	protected $properties;
	
	/**
	 * Form class constructor.
	 * 
	 * Initializes a Form object.
	 * 
	 * @param string $title Title (or caption) for the form.
	 * @param string $name Name of this form.
	 * @param string $action URL where form gets posted.
	 * @param array $fields One or more Field objects.
	 * @param array $buttons One or more Button objects.
	 * @param PropertyHolder $style Extra style hints for the form.
	 */
	public function __construct($title, $name, $action, $fields, $buttons, $properties = null, $style = false)
	{
		// Make sure these variables are arrays.
		if (!is_array($fields))  $fields  = array ($fields);
		if (!is_array($buttons)) $buttons = array ($buttons);

		// Test all the fields and buttons and make sure they
		// are of the proper type (Field or Button class objects).
		foreach ($fields as $fld)
			if (!($fld instanceof Field))
				throw new Exception("Form Error: Not all fields are of type 'Field'!");
		
		foreach ($buttons as $btn)
			if (!($btn instanceof Button))
				throw new Exception("Form Error: Not all buttons are of type 'Button'!");

		$this->properties = new PropertyBag(
			array(
				"title" => $title,
				"name" => $name,
				"action" => $action,
				"fields" => $fields,
				"buttons" => $buttons,
				"style" => false
			),
			$properties);

		if ($style)
			if ($style instanceof PropertyHolder)
				$this->style = $style;
			else
				throw new Exception("Form constructor error: \$style is of type (".get_class($style).") which does not implement the PropertyHolder interface.");
	}

	/**
	 * Property accessor.
	 * 
	 * @param string $name Name of the property.
	 */	
	public function __get($name)
	{
		return $this->properties->$name;
	}
	
	/**
	 * Property mutator.
	 * 
	 * @param string $name Name of the property.
	 * @param any $value Value to assign to the property.
	 */
	public function __set($name,$value)
	{
		$this->properties->$name = $value;
	}
	
	/**
	 * Send the form to the user.
	 * 
	 * This function generates the HTML code for the form.  The
	 * form will use a 2 column table to format the labels and
	 * fields.  Also outputs the buttons as the last row of the
	 * table.
	 */
	final public function display()
	{
		$this->formStart($this->style);
		$this->formHeader($this->style);

		foreach ($this->fields as $fld)
			if (!($fld instanceof HiddenField))
				$fld->display($this->style);
		
?>
<tr><tr><td colspan="2" align="center">
<?php
		for ($i = 0; $i < count($this->buttons); ++$i) {
			$this->buttons[$i]->display($this->style);
			if ($i < (count($this->buttons)-1))
				echo '&nbsp;&nbsp;';
		}
?>
</td></tr></tr>
<?php
		
		$this->formFooter($this->style);
		
		foreach ($this->fields as $fld)
			if ($fld instanceof HiddenField)
				$fld->display($this->style);
				
		// Append the form name in a hidden 'form-name' field.
		$formField = new HiddenField("form-name", get_class($this) );
		$formField->display($this->style);
	
		$this->formEnd($this->style);
	}
	
	/**
	 * Search for a field by name and return it.
	 * 
	 * @param string $fieldName The name of the field to search for.
	 * @return Field The field if found, false if not found.
	 */
	public function getField($fieldName)
	{
		foreach ($this->fields as $fld)
			if ($fld->name === $fieldName)
				return $fld;
		return false;
	}
	
	/**
	 * The default implementation of formStart() begins the form
	 * and starts an HTML table.
	 */
	protected function formStart($style)
	{
		echo "<form name='$this->name' action='$this->action' method='POST'>\n";
		echo "<table border='0'>\n";
	}
	
	/**
	 * The default implementation of formHeader() outputs the form
	 * title in a row that spans 2 columns of the HTML table.
	 */
	protected function formHeader($style)
	{
		echo "<tr><td colspan='2' align='center'><b>$this->title</b></td></tr>\n";
	}

	/**
	 * The default version of formFooter() closes the table.
	 */
	protected function formFooter($style)
	{	
		echo "</table>\n";
	}
	
	/**
	 * The default version of formEnd() closes the form.
	 */
	protected function formEnd($style)
	{
		echo "</form>\n";
	}
	
	public function isValid()
	{
		foreach ($this->fields as $fld)
			if (!$fld->isValid())
				return false;
		return true;
	}
}

?>
