<?php
/**
 * Defines the Listbox class used on forms.
 * 
 * @package FormLib
 */
  
/**
 * An object-oriented wrapper around an HTML listbox.
 * 
 * You must supply either an array or a PropertyHolder object with the
 * list of items to be added to the listbox.  This class will always
 * make the first item in the list "--Please Select--" with a value
 * of -1.  This can be suppressed by setting the showPleaseSelect
 * property to false after constructing the Listbox object.
 * 
 * Here is an example:
 * <code>
 * <?php
 *   $mybox = new Listbox("mybox", "Select Color:",
 *       array('red' => "Bright Red", "blue" => "Baby Blue", "green" => "Lime Green"));
 * ?>
 * </code>
 * 
 * Another example using a PropertyBag object:
 * <code>
 * <?php
 *   $myitems = new PropertyBag(
 *       "red" => "Bright Red",
 *       "blue" => "Baby Blue",
 *       "green" => "Lime Green");
 *   $mybox = new Listbox("mybox", "Select Color:", $myitems);
 * ?>
 * </code>
 * 
 * When the user selects one of the items, the <em>value</em> property will
 * be set to the value.  For example, if the user chooses the 'Bright Red'
 * option, the <em>value</em> property will be set to 'red'.
 * 
 * @package FormLib
 * @version 0.1
 * @author Randy
 */

class Listbox extends Field
{
	/**
	 * Constructor.
	 * 
	 * @param string $name Name assigned to the list box.
	 * @param string $label Label for the list box.
	 * @param array/PropertyHolder $options List of value/name pairs.
	 * @param any $value The selected item in the list.
	 * @param PropertyHolder $properties Extra properties to attach to the list box.
	 * @param PropertyHolder $style Additional style hinting information.
	 */
	public function __construct($name, $label, $options, $value = "", $style = false)
	{
		parent::__construct($name, $label, $value, false, $style);
		
		if (is_array($options) || ($options instanceof PropertyHolder))
			$this->properties->add("options", $options);
		else
			throw new Exception("Listbox __construct error: \$options must be an array or PropertyHolder object!");
		
		// Add optional onChange property for holding JavaScript code
		$this->properties->add("onChange", false);
		$this->properties->add("showPleaseSelect", true);
	}
	
	/**
	 * Renders the list box as HTML.
	 * 
	 * @param PropertyHolder $formStyle The form's style options. (Not used)
	 */
	protected function displayField($formStyle)
	{
		$js = $this->onChange ? " onchange=\"$this->onChange\"" : "";
		
		echo "<select name=\"$this->name\" $js>\n";
		if ($this->showPleaseSelect)
			echo " <option value=\"-1\">-- Please Select --</option>\n";
			
		if (is_array($this->options)) {
			foreach ($this->options as $value => $label) {
				echo ' <option value="';
				p ($value);
				echo '"';
				if( $this->value === $value)
					echo ' selected="true"';
				echo '>';
				p ($label === "" ? $value : $label);
				echo "</option>\n";
			}
		} else {
			foreach ($this->options->names() as $value) {
				echo ' <option value="';
				p ($value);
				echo '"';
				if ($this->value == $value)
					echo ' selected="true"';
				echo '>';
				p ($this->options->$value);
				echo "</option>\n";
			}
		}
		echo "</select>\n";			
	}
}

?>
