<?php
/**
 * Defines the PropertyBag class.
 * 
 * @package SupportLib
 */
 
/**
 * An implementation of a PropertyHolder that uses an array to store
 * name/value property pairs.
 * 
 * @package SupportLib
 * @version 0.2
 * @author Randy L. Pearson
 */
class PropertyBag implements PropertyHolder
{
	private $properties;
	
	/**
	 * Constructor.
	 * 
	 * This version improves over previous versions by allowing the object
	 * to be initialized by copying properties from either arrays or other
	 * PropertyHolder objects.
	 * 
	 * @param array $properties An array holding a list of valid property name/value pairs.
	 * @param array $extra A second array of optional values that will be merged into the properties.
	 */
	
	public function __construct($properties)
	{
		if ($properties instanceof PropertyHolder) {
			$properties = array();
			foreach ($properties->names() as $field)
				$this->add($field, $properties->$field);
		} elseif (is_array($properties)) {
			$this->properties = $properties;
		} else {
			throw new Exception("Cannot initialize properties from type ".get_class($properties));
		}
	}
	
	/**
	 * Accessor function that reads a property's value.
	 * 
	 * @param string $name Name of the property to be returned.
	 * 
	 * @return any Throws error on invalid property name.
	 */
	public function __get($name)
	{
		$this->checkName($name);
		return $this->properties[$name];
	}
	
	/**
	 * Mutator function that updates a property's value.
	 * 
	 * @param string $name Name of the property to be updated.
	 * @param any $value The value to assign.
	 * 
	 * @return none Throws error on invalid property name.
	 */
	public function __set($name, $value)
	{
		$this->checkName($name);
		$this->properties[$name] = $value;
	}
	
	/**
	 * Determines if a name is valid.
	 * 
	 * @return none Throws error if not valid.
	 */
	protected function checkName($name)
	{
		if (!array_key_exists($name, $this->properties)) {
			throw new Exception("Invalid Property Name: '$name'");
		}
	}
	
	/**
	 * Adds a new property to the bag.
	 * 
	 * @param string $name Name of the new property.
	 * @param string $value Value of the property.  (Default: "")
	 */
	public function add($name, $value = "")
	{
		if(in_array($name, $this->properties)) {
			throw new Exception("Property Already Exists: '$name'");
		} else {
			$this->properties[$name] = $value;
		}
	}
	
	/**
	 * Returns a list of valid property names.
	 * 
	 * @return array An array of the current property names as strings.
	 */
	public function names()
	{
		return array_keys($this->properties);
	}
	
	/**
	 * Tests for a valid property name.
	 * 
	 * @since 0.2 - Nov 6, 2006
	 */
	public function isValidName($name)
	{
		return in_array($name, array_keys($this->properties));
	}
}
?>
