<?php
/**
 * This file defines a number of helper functions that make it
 * easier to query, insert, update or delete database records.
 * 
 * It uses the PDO classes provided with PHP 5.x as the low level
 * database abstraction layer.  Prior to calling any of the functions
 * in this file, you must establish a database connection and store
 * the connection in a global variable named <code>$db</code>.
 */

require_once dirname(__FILE__) . DIRECTORY_SEPARATOR . "weblib.php";

/**
 * Record a PDO error message to the log file.
 * 
 * @param PDOException $error A PDO exception object.
 */

function reportError($error)
{
	global $CFG;
	
	$msg = 'Database Error '.$error->getCode().': '.$error->getMessage();
	if (isset($CFG->debug) and $CFG->debug > 7)
		notify($msg);
	error_log($msg);
}

/**
 * Executes the provided SQL command and optionally returns the results.
 * 
 * @param string $command The SQL command to execute
 * @param boolean $wantResult Flag to return the results
 * @param boolean $feedback Flag to show result to user
 * @param boolean $dieOnError Flag to abort script on failure
 * @return any Returns true/false to indicate success/failure or the results if
 * <code>$wantResult</code> is true
 */

function execute_sql($command, $wantResult=false, $feedback=false, $dieOnError=false)
{
	global $db;

	try {	
		$result = $db->query($command);
		if($wantResult) {
			return $result;
		}
	} catch (PDOException $e) {
		if($feedback)
			echo "<p><font color=\"red\"><b>Error: ".$e->getMessage()."</b></font></p>";
		if($dieOnError)
			die;
		return false;
	}
		
	if($feedback)
		echo "<p><font color=\"green\"><b>Success</b></font></p>";
	return true;
}

/**
 * Determine if a record matching up to 3 filter conditions exists or not.
 * 
 * @param string $table The database table/view name
 * @param array $filter An array of field name/value pairs.
 * @return boolean True indicates one or more records exist
 */

function record_exists($table, $filter)
{
	return (count_records($table, $filter) > 0);
}

/**
 * Converts name/values pairs into an an SQL where clause.
 * 
 * @param array Array of name/value pairs.
 * @param &$select References a string to hold the query.
 * @param &$values References an array to hold the parameters.
 */
function build_filter($filter, &$values)
{
	$select = "";

	if (is_array($filter)) {	
		foreach ($filter as $field => $value) {
			if (!empty($select))
				$select .= ' AND ';
			else
				$select = 'WHERE ';
				
			$select .= "$field = ?";
			$values[] = $value;
		}
	}
	
	return $select;
}

/**
 * Returns the number of records that match the conditions.
 * 
 * Parameters:
 * 
 * @param string $table  The name of the database table
 * @param array $filter An array of field name/value pairs.
 * @return integer Number of records that match the filter conditions.
 */

function count_records($table, $filter="")
{
	global $db;
	
	$values = array();
	
	$sql = "SELECT COUNT(*) FROM $table " . build_filter($filter, $values);
	
	try {
		$stmt = $db->prepare($sql);
		if ($stmt->execute($values)) {
			$rec = $stmt->fetch(PDO::FETCH_NUM);
			$stmt->closeCursor();	// Some databases expect this
			if (is_array($rec))
				return $rec[0];
		}
	} catch (PDOException $e) {
		reportError($e);
	}
	return 0;
}

/**
 * Lookups a record using a filter and returns it as a standard object.  If more
 * than one record matches, only the first is returned.
 * 
 * @param string $table The name of the database table
 * @param array $filter An array of field name/value pairs.
 * @return object The first matching record, or false if not found.
 */

function get_record($table, $filter)
{
	global $db;
	
	$values = array();
	
	$sql = "SELECT * FROM $table " . build_filter($filter, $values);
	
	try {
		$stmt = $db->prepare($sql);
		if ($stmt->execute($values)) {
			$rec = $stmt->fetch(PDO::FETCH_OBJ);
			$stmt->closeCursor();	// Some databases expect this
			return $rec;
		}
	} catch (PDOException $e) {
		reportError($e);
	}
	return false;
}

/**
 * Gets zero or many records from a table as an array of objects with optional
 * filtering, sorting, list of fields and limits.
 * 
 * <b>NOTE:</b> This function can be more efficient by using the native
 * database LIMIT/OFFSET options, however they are not portable and the
 * details differ from one database server to another.  This function will
 * use a lot of memory on large datasets.
 * 
 * @param string $table The name of the database table
 * @param array $filter An array of field name/value pairs.
 * @param string $sort  The optional sort sequence.
 * @param string $fields An optional list of record fields.
 * @param integer $limitfrom  An optional offset to the first record.
 * @param integer $limitcount An optional limit on number of records returned.
 * @return object The first matching record, or false if not found.
 */

function get_records($table, $filter="", $sort="", $fields="*", $limitfrom=0, $limitcount=0)
{
	global $db;
	
	$values = array();
	
	$sql = "SELECT $fields FROM $table " . build_filter($filter, $values);
	
	if(!empty($sort))
		$sql .= " ORDER BY $sort";
	
	try {
		$stmt = $db->prepare($sql);
		if ($stmt->execute($values)) {
			$records = $stmt->fetchAll(PDO::FETCH_OBJ);
			
			if ($limitcount > 0)
				return array_slice($records, $limitfrom, $limitcount);
			else
				return $records;
		}
	} catch (PDOException $e) {
		reportError($e);
	}
	return false;
}

/**
 * Get a single field from a record.
 * 
 * @param string $table The name of the database table
 * @param array $filter An array of field name/value pairs.
 * @param $return The field/column value to return.
 * @return any The value on success, <code>null</code> otherwise.
 */

function get_field($table, $return, $filter)
{
	global $db;
	
	$values = array();
	
	$sql = "SELECT $return FROM $table " . build_filter($filter, $values); 

	try {
		$stmt = $db->prepare($sql);
		if ($stmt->execute($values)) {
			return $stmt->fetchColumn(0);
		}
	} catch (PDOException $e) {
		reportError($e);
	}
	return null;
}

/**
 * Updates one field in a database record.
 * 
 * @param string $table The name of the database table
 * @param string $setfield The field in the record to change
 * @param any $newvalue The field's new value
 * @param array $filter An array of field name/value pairs
 * @return boolean True indicates success
 */
function set_field($table, $setfield, $newvalue, $filter)
{
	global $db;
	
	$values[] = $newvalue;
	
	$sql = "UPDATE $table SET $setfield = ? " . build_filter($filter,$values);
	
	try {
		$stmt = $db->prepare($sql);
		return $stmt->execute($values);
	} catch (PDOException $e) {
		reportError($e);
	}
	return false;
}

/**
 * Deletes one or more database records
 * 
 * @param string $table The name of the database table
 * @param array $filter An array of field name/value pairs
 * @return boolean True indicates success
 */
function delete_records($table, $filter)
{
	global $db;

	if (!isset($filter) or empty($filter) or !is_array($filter))
		throw new Exception("Cowardly refusing to delete all records!");
			
	$values = array();
	
	$sql = "DELETE FROM $table " . build_filter($filter, $values);
	
	try {
		$stmt = $db->prepare($sql);
		return $stmt->execute($values);
	} catch (PDOException $e) {
		reportError($e);
	}
	return false;
}

/**
 * Helper function to retrieve information about a table's columns.
 * 
 * @param string $table Name of the table in the database.
 * @return array An array of column metadata (See PDO documentation.)
 */
function table_info($table)
{
	global $db;
	
	$sql = "SELECT * FROM $table";
	try {
		if ($result = $db->query($sql)) {
			for ($col = 0; $col < $result->columnCount(); ++$col)
				$columns[] = $result->getColumnMeta($col);
			$result->closeCursor();  // Some databases expect this
			return $columns;
		}
	} catch (PDOException $e) {
		reportError($e);
	}
	return array();
}

/**
 * Creates a new database record from an object.
 * 
 * @param string $table The name of the database table
 * @param object $dataobject A standard object holding the record values
 * @param boolean $returnid Flag to return the inserted record's primary key
 * @return boolean Return <code>true</code> on success, unless
 * <code>$returnid</code> is true, then returns the new ID value.
 */
function insert_record($table, $dataobject, $returnid=false)
{
	global $CFG, $db;
	
	// First, we need to know the table's columns
	if (!($colInfo = table_info($table))) {
		reportError("Failed to get table column information for table $table");
		return false;
	}
	
	// Now we check to see if the data object has a
	// value for each column in the record.  If so,
	// store the name and value for later usage.
	foreach($colInfo as $info) {
		if(isset($dataobject->$info['name'])) {
			$fields[] = $info['name'];
			$params[] = '?';
			$values[] = $dataobject->$info['name']; 
		}
	}
	
	// Abort if no data was found to insert
	if(count($fields) == 0)
		return false;
		
	// Build the SQL statement we need
	$sql = "INSERT INTO $table (";
	$sql .= implode(",", $fields);
	$sql .= ") VALUES (";
	$sql .= implode(",",$params);
	$sql .= ")";

	try {
		$stmt = $db->prepare($sql);
		if ($stmt->execute($values)) {
			// Not all  databases/tables support this
			return ($returnid ? $db->lastInsertId() : true);
		}
	} catch (PDOException $e) {
		reportError($e);
	}
	return false;
}

/**
 * Issues an UPDATE statement with modified record values.
 * 
 * @param string $table Name of the database table.
 * @param object $dataobject A PHP object with the new record values.
 * @param string $primarykey The name of the primary key column.
 * @return boolean Indicates success or failure.
 */
function update_record($table, $dataobject, $primarykey="id")
{
	global $CFG, $db;
	
	// First, make sure we have the primary key value
	if(!isset($dataobject->$primarykey) or empty($dataobject->$primarykey))
		return false;
	
	// Next we need the names of the columns in the table
	if (!($colInfo = table_info($table))) {
		reportError("Failed to get table column information for table $table");
		return false;
	}
	
	// Now we check to see if the data object has a
	// value for each column in the record.  If so,
	// store the name and value for later usage.
	foreach($colInfo as $info) {
		if($info["name"] != $primarykey && isset($dataobject->$info["name"])) {
			$fields[$info["name"]] = $dataobject->$info["name"];
		}
	}
	
	// Abort if nothing found to update
	if(count($fields) == 0)
		return false;
	
	// Prepare the SET clause
	$numflds = count($fields);
	$count = 0;
	$update = "";
	
	foreach($fields as $col => $val) {
		$count++;
		$update .= "$col = ?";
		$values[] = $val;
		if($count < $numflds)
			$update .= ", ";
	}

	// Build the final SQL statement with a WHERE clause
	$sql = "UPDATE $table SET $update WHERE $primarykey = ?";
	$values[] = $dataobject->$primarykey;

	// Perform the record update now
	try {
		$stmt = $db->prepare($sql);
		return $stmt->execute($values);
	} catch (PDOException $e) {
		reportError($e);
	}
	return false;
}

/**
 * Queries the database and displays links to jump to a restrict set of records
 * based on the first character of a field in the table.
 * 
 * The query extracts only the first character from each column and then builds
 * a URL by appending "letter=x" to the supplied URL.
 * 
 * @param string $table The name of the table to query.
 * @param string $column The column to be searched.
 * @param string $baseurl The URL that points to the linked page.
 */
function quick_links($table, $column, $baseurl)
{
	$links = "<a href=\"$baseurl\">All</a>";
	$result = execute_sql("SELECT DISTINCT LEFT($column,1) AS letter FROM $table ORDER BY letter", true);

	foreach ($result->fetchAll(PDO::FETCH_OBJ) as $record) {
		if (trim($record->letter)) {
			$links .= "\n| ";
			$letter = urlencode($record->letter);
			$param = (strpos($baseurl, '?') > 0 ? '+' : '?');
			$links .= '<a href="' . $baseurl . $param . "letter=$letter\">$record->letter</a>";
		}
	}

	echo str_repeat("&nbsp;", 4) . "\n" . $links . str_repeat("&nbsp;", 4) . "\n";
}

?>