<?php

/*
 * Created on Sep 12, 2006
 *
 * weblib.php -- Web related support functions
 */

/**
 * Extra support functions that make it a bit easier to handle
 * various HTML/web operations and gives access to different
 * page and/or script elements and variables. 
 */

/**
 * Returns the variable if defined, a default value if not.
 * 
 * @param any $var - Reference to the variable to test
 * @param any $default - Value to return if the variable is not set
 * @return any The variable or default value.
 */

function nvl(& $var, $default = "")
{
	return isset ($var) ? $var : $default;
}

/**
 * Improved version of the stripslashes() function.
 * 
 * The original strips out all backslashes, so strings like
 * <code>c:\temp</code> becomes <code>c:temp</code>.
 * 
 * @param string $string The string to strip.
 * @return string The stripped string.
 */

function stripslashes_safe($string)
{
	if (!in_array(gettype($string), array("string","boolean","integer","double")))
		throw new Exception("Cannot strip slashes from type ".gettype($string));
		
	$string = str_replace("\\'", "'", $string);
	$string = str_replace('\\"', '"', $string);
	$string = str_replace('\\\\', '\\', $string);

	return $string;
}

/**
 * Removes the query parameters from a URL
 * 
 * @param string $url - the URL to test
 * @return string The URL with the query removed.
 */

function strip_query($url)
{
	if ($pos = strpos($url, '?'))
		return substr($url, 0, $pos);
	return $url;
}

/**
 * Return an empty string or quotes special characters like "<" and ">"
 * 
 * @param string $var - the string to process
 * @return string Empty string or processed string.
 */

function s($var)
{
	if (empty ($var))
		return "";
	return htmlSpecialChars(stripslashes_safe($var));
}

/**
 * Prints a variable safely.
 * 
 * @param any $var The variable to print.
 */

function p($var)
{
	echo s($var);
}

/**
 * Get the current script URI, include the query.
 * 
 * @return string The URI.
 */

function me()
{
	// Try the "REQUEST_URI" variable first
	if (!empty ($_SERVER['REQUEST_URI']))
		return $_SERVER['REQUEST_URI'];

	// Try "PHP_SELF" next
	if (!empty ($_SERVER['PHP_SELF']))
		if (!empty ($_SERVER['QUERY_STRING']))
			return $_SERVER['PHP_SELF'] . '?' . $_SERVER['QUERY_STRING'];
		else
			return $_SERVER['PHP_SELF'];

	// Last resort, fallback on "SCRIPT_NAME"
	if (!empty ($_SERVER['SCRIPT_NAME']))
		if (!empty ($_SERVER['QUERY_STRING']))
			return $_SERVER['SCRIPT_NAME'] . '?' . $_SERVER['QUERY_STRING'];
		else
			return $_SERVER['SCRIPT_NAME'];

	notify("Warning: Unable to determine script URI.  Is your PHP installed correctly?");
	return false;
}

/**
 * Sends a notification message to the browser.
 *
 * Parameters:
 *
 * $message - The message.
 * $color   - The color.
 * $align   - An alignment setting.
 */

function notify($message, $color = "red", $align = "center")
{
	echo "<p align=\"$align\"><b><font color=\"$color\" size=\"+1\">$message</font></b></p>\n";
}

/**
 * Get the complete URL of the current script including protocol and
 * server name.
 * 
 * @return string The current URL.
 */

function full_me()
{
	if (!empty ($_SERVER['HTTP_HOST']))
		$hostname = $_SERVER['HTTP_HOST'];
	elseif (!empty ($_ENV['HTTP_HOST'])) $hostname = $_ENV['HTTP_HOST'];
	elseif (!empty ($_SERVER['SERVER_NAME'])) $hostname = $_ENV['SERVER_NAME'];
	elseif (!empty ($_ENV['SERVER_NAME'])) $hostname = $_ENV['SERVER_NAME'];
	else
	{
		notify("Warning: Unable to determine server's name!");
		return false;
	}

	$protocol = (isset ($_SERVER['HTTPS']) and $_SERVER['HTTPS'] == "on") ? "https://" : "http://";

	return $protocol . $hostname . me();
}

/**
 * Starts a new page.
 * 
 * First looks for a file named <code>$CFG->dirroot/header.php</code>.  If
 * found, the function includes it.  If not found, the function outputs a generic
 * HTML header.
 * 
 * @param string $title Title of the page.
 * @param string $heading Optional heading for the page.
 * @param string $navigation Optional navigation links/breadcrumb.
 * @param string $focus Optional field to focus on (using JavaScript).
 * @param string $meta Optional additional meta tags.
 * @param boolean $cache Flag to enable/disable browser caching
 * @param boolean $button Optional button to create.
 * @param string $menu  Optional menu (HTML) to add to page.
 */
function print_header($title = "", $heading = "", $navigation = "", $focus = "", $meta = "", $cache = true, $button = "&nbsp;", $menu = "")
{
	global $CFG, $SESSION;

	if (isset($CFG->header_sent) && $CFG->header_sent)
		return;
	
	$CFG->header_sent = 1;
	
	if ($button == "")
		$button = "&nbsp;";

	if (isset ($CFG->debug) and $CFG->debug > 7)
		$cache = false;

	// Try to disable caching if requested.
	if (!$cache)
	{
		@ header("Last-Modified: " . gmdate("D, d M Y H:i:s") . " GMT");
		@ header("Pragma: no-cache");
		$meta .= "\n<meta http-equiv=\"pragma\" content=\"no-cache\" />";
		$meta .= "\n<meta http-equiv=\"expires\" content=\"0\" />";
	}

	$title = s($title);

	if(file_exists("$CFG->dirroot/header.php"))
		include ("$CFG->dirroot/header.php");
	else {
?>
<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN" "http://www.w3.org/TR/html4/loose.dtd">
<html>
<head>
  <title><?php echo $title; ?></title>
  <meta http-equiv="Content-Type" content="text/html; charset=iso-8859-1">
</head>
<?php
	
		// Handle optional focus on specific control
		//
		// NOTE: This should also be in header.php!!!
		//
		if(!empty($focus)) {
			echo '<body onload=setfocus()>';
			echo '<script language="JavaScript">';
			echo '<!-- hide from older browsers';
			echo 'function setfocus() { document.'.$focus.'.focus() }\n';
			echo '//done hiding-->';
			echo '</script>';
		} else {
			echo '<body>';
		}
	}
	
	if (!empty ($navigation))
		print_navigation($navigation);
		
	if (!empty($menu))
		echo $menu;
}

/**
 * Ends the current page.
 * 
 * First looks for a <code>$CFG->footer.php</code> file and includes
 * it if found, otherwise output a default set of closing tags.
 */
function print_footer()
{
	global $CFG;

	if(file_exists("$CFG->dirroot/footer.php"))
		include ("$CFG->dirroot/footer.php");
	else {
?>
</body>
</html>
<?php
	}
}

/**
 * Outputs an HTML meta refresh tag to send the visitor to a new URL.
 * 
 * @param string $url The new URL.
 * @param string $message An optional message to be displayed.
 * @param integer $delay Number of seconds to show message.
 */

function redirect($url, $message = "", $delay = "0", $color = "")
{
	if (empty ($message))
	{
		echo "<meta http-equiv=\"refresh\" content=\"$delay; url=$url\" />";
		echo "<script>location.replace('$url');</script>"; // Mozilla bug
	} else
	{
		if (empty ($delay))
			$delay = 3; // Why have message with no delay?

		print_header("", "", "", "<meta http-equiv=\"refresh\" content=\"$delay; url=$url\" />");
		echo "<p align=\"center\">";
		if (!empty($color))
			echo "<font color=\"$color\" size=\"+1\">";
		p($message);
		if (!empty($color))
			echo "</font>";
		echo "<br/><br/>( <a href=\"$url\">Click if not redirected in $delay seconds</a> )";
		echo "</p>";
		print_footer();
		flush();
		sleep($delay);
		echo "<script>location.replace('$url');</script>"; // Mozilla bug
	}
	die;
}

/**
 * Returns the HTTP_REFERER without the query parameters.
 *
 * @param boolean $withQuery Flag to return/strip query parameters.
 * @return string The referer URL. 
 */

function get_referer($withQuery = false)
{
	if ($withQuery)
		return nvl($_SERVER['HTTP_REFERER']);
	else
		strip_query(nvl($_SERVER['HTTP_REFERER']));
}

/**
 * Adds previous and next page links to the bottom of a table as needed.
 * 
 * @param integer $currPage The current page number.
 * @param integer $totalPages The total number of pages.
 * @param string $prevPageLink The URL to the previous page.
 * @param string $nextPageLink The URL to the next page.
 * @param integer $tableCols The number of columns in the containing table.
 */
function show_page_links($currPage, $totalPages, $prevPageLink, $nextPageLink, $tableCols)
{
	if ($totalPages == 1) return;
?>
<tr><td colspan="<?php echo $tableCols; ?>">&nbsp;</td></tr>
<tr><td colspan="<?php echo $tableCols; ?>"><table border="0" width="100%"><tr>
<?php
	if ($currPage > 1 && $currPage < $totalPages) {
?>
<td align="left"><a href="<?php echo $prevPageLink; ?>">Prev Page</a></td>
<td align="right"><a href="<?php echo $nextPageLink; ?>">Next Page</a></td>
<?php
	} else if ($currPage < $totalPages) {
?>
<td align="right"><a href="<?php echo $nextPageLink; ?>">Next Page</a></td>
<?php
	} else if ($currPage > 1) {
?>
<td align="left"><a href="<?php echo $prevPageLink; ?>">Prev Page</a></td>
<?php
	}
?>
</tr></table></td></tr>
<?php
}
?>